# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import pandas as pd
from cn2an import transform
try:
    from ..utils import get_file_path
except:
    from chncal.utils import get_file_path


constants_template = """# -*- coding: utf-8
    
# this file is generated by:
# chncal.constant_creator.chengming.create_chengming_constant

from __future__ import absolute_import, unicode_literals


w_year = {}

w_month = {}

w_date = {}

w_hour = {}

song = {}
"""


def load_chengming():

    def _trans_weight(df):
        df['重量'] = df['重量'].apply(lambda x: transform(x.replace('两', '.')))
        df['重量'] = df['重量'].apply(lambda x: 
                     x.replace('钱', '') if '.' in x else '0.'+x.replace('钱', ''))
        df['重量'] = df['重量'].apply(lambda x: str(round(float(x), 2)))
        return df    
    
    def _handle_year(df_year):
        df = df_year.copy()
        df['干支']  = df['干支'] + '(' + df['属相'] + ')'
        df.drop('属相', axis=1, inplace=True)
        df = _trans_weight(df)
        return df    
    
    def _handle_month(df_month):
        df = df_month.copy()
        df['月份'] = df['月份'].apply(lambda x: x.replace('正', '一').replace('冬', '十一').replace('腊', '十二'))
        df['月份'] = df['月份'].apply(lambda x: transform(x.replace('月', '')))
        df['月份'] = df['月份'].apply(lambda x: str(x).zfill(2))
        df = _trans_weight(df)
        return df    
    
    def _handle_date(df_date):
        df = df_date.copy()
        df['日期'] = df['日期'].apply(lambda x: x.replace('初', '').replace('廿', '二十'))
        df['日期'] = df['日期'].apply(transform)
        df['日期'] = df['日期'].apply(lambda x: str(x).zfill(2))
        df = _trans_weight(df)
        return df    
    
    def _handle_hour(df_date):
        df = df_date.copy()
        df['地支'] = df['地支'].apply(lambda x: x.replace('时', ''))
        df = _trans_weight(df)
        return df    
    
    def _handle_song(df_song):
        df = df_song.copy()
        df['重量'] = df['重量'].apply(lambda x: x if x.endswith('两') else x+'钱')
        df = _trans_weight(df)
        return df
    
    fpath = get_file_path('chengming.xlsx', 'data')
    
    df_year = pd.read_excel(fpath, sheet_name='干支年')
    w_year = _handle_year(df_year)
    w_year = w_year.set_index('干支')['重量'].to_dict()
    
    
    df_month = pd.read_excel(fpath, sheet_name='月(农历)')
    w_month = _handle_month(df_month)
    w_month = w_month.set_index('月份')['重量'].to_dict()
    
    
    df_date = pd.read_excel(fpath, sheet_name='日(农历)')
    w_date = _handle_date(df_date)
    w_date = w_date.set_index('日期')['重量'].to_dict()
    
    
    df_hour = pd.read_excel(fpath, sheet_name='时辰')
    w_hour = _handle_hour(df_hour)
    w_hour = w_hour.set_index('地支')['重量'].to_dict()
    
    
    df_song = pd.read_excel(fpath, sheet_name='袁天罡称命歌')
    df_song = _handle_song(df_song)
    song = df_song.set_index('重量')['歌诀'].to_dict()
    
    return w_year, w_month, w_date, w_hour, song


def _get_lines(d: dict):
    yield '{'
    line = "    '{}': '{}',"
    for k, v in d.items():
        yield line.format(k, v)
    yield '}'


def create_chengming_constant():
    w_year, w_month, w_date, w_hour, song = load_chengming()
    
    year_str = '\n'.join(_get_lines(w_year))
    month_str = '\n'.join(_get_lines(w_month))
    date_str = '\n'.join(_get_lines(w_date))
    hour_str = '\n'.join(_get_lines(w_hour))
    song_str = '\n'.join(_get_lines(song))
    
    file_content = constants_template.format(year_str, month_str, date_str, hour_str, song_str)
    
    fate_path = get_file_path('chengming.py')    
    with open(fate_path, 'wb') as f:
        f.write(file_content.encode('utf-8'))


if __name__ == '__main__':
    create_chengming_constant()
